"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const validator_1 = require("validator");
const Handler_1 = require("./Handler");
exports.Validators = Object.assign(validator_1.default, {
    Builders: {
        range({ min, max }) {
            return (val) => val >= min && val <= max;
        },
        oneOf({ vals }) {
            return (val) => (vals || []).includes(val);
        },
        equals({ val: expectedValue }) {
            return (val) => {
                if (_.isFunction(expectedValue)) {
                    return this.equals({ val: expectedValue(val) });
                }
                return _.isEqual(expectedValue, val);
            };
        },
    },
    invoke(parserMiddleware, ...params) {
        return Handler_1.Handler.invokeOrHandle(parserMiddleware, params, exports.Validators, true);
    },
    isEthTransactionId(value) {
        return this.isHexadecimal(value) && value.length == 64;
    },
    isEthPrivateKey(value) {
        return this.isHexadecimal(value) && value.length == 128;
    },
    isEthPublicKey(value) {
        return this.isHexadecimal(value) && value.length == 64;
    },
});
//# sourceMappingURL=Validators.js.map